Package: COUNT
Type: Package
Title: Functions, Data and Code for Count Data
Version: 1.3.5
Date: 2025-04-27
Imports: MASS
Depends: R (>= 2.10), msme, sandwich
Authors@R: c(person(given = c("Joseph", "M"),
                      family = "Hilbe",
                      role = "aut",
                      email = "hilbe@asu.edu"),
               person(given = "Andrew",
                      family = "Robinson",
                      role = "cre",
                      email = "apro@unimelb.edu.au"))
Description: Functions, data and code for Hilbe, J.M. 2011. Negative Binomial Regression, 2nd Edition (Cambridge University Press) and Hilbe, J.M. 2014. Modeling Count Data (Cambridge University Press).
License: GPL-2
LazyLoad: yes
NeedsCompilation: no
Packaged: 2025-04-27 10:40:03 UTC; apro
Author: Joseph M Hilbe [aut],
  Andrew Robinson [cre]
Maintainer: Andrew Robinson <apro@unimelb.edu.au>
Repository: CRAN
Date/Publication: 2025-04-28 10:00:27 UTC
Built: R 4.3.3; ; 2025-04-28 11:37:08 UTC; unix
